@@ -59,7 +59,11 @@ def getLevel(name):
 
 CONFIG_FILE = '/etc/ipahealthcheck/ipahealthcheck.conf'
 CONFIG_SECTION = 'default'
+EXCLUDE_SECTION = 'excludes'
+
+DEFAULT_TIMEOUT = 10
 
 DEFAULT_CONFIG = {
     'cert_expiration_days': 28,
+    'timeout': DEFAULT_TIMEOUT,
 }
