@@ -29,6 +29,7 @@ from ipaserver.plugins import ldap2
 from ipapython import certdb
 from ipapython import ipautil
 from ipapython.dn import DN
+from ipapython.ipaldap import realm_to_serverid
 
 logger = logging.getLogger()
 DAY = 60 * 60 * 24
@@ -145,6 +146,23 @@ def get_expected_requests(ca, ds, serverid):
     else:
         logger.debug('No KDC pkinit certificate')
 
+    # See if a host certificate was issued. This is only to
+    # prevent a false-positive if one is indeed installed.
+    local = {
+        paths.IPA_NSSDB_DIR: 'Local IPA host',
+        paths.NSS_DB_DIR: 'IPA Machine Certificate - %s' % socket.getfqdn(),
+    }
+    for db, nickname in local.items():
+        nssdb = certdb.NSSDatabase(db)
+        if nssdb.has_nickname(nickname):
+            requests.append(
+                {
+                    'cert-database': db,
+                    'cert-nickname': nickname,
+                    'ca-name': 'IPA',
+                }
+            )
+
     return requests
 
 
@@ -373,7 +391,7 @@ class IPACertTracking(IPAPlugin):
                              key=request_id,
                              error=str(e),
                              msg='Found request id {key} but it is not tracked'
-                                 'by certmonger!?: {error}')
+                                 ' by certmonger!?: {error}')
                 continue
 
             # The criteria was not met
@@ -418,10 +436,14 @@ class IPACertDNSSAN(IPAPlugin):
         for request in requests:
             request_id = certmonger.get_request_id(request)
             if request_id is None:
-                yield Result(self, constants.ERROR,
-                             key=request_id,
-                             msg='Found request id {key} but it is not tracked'
-                                 'by certmonger!?')
+                # log and skip. Missed tracking is reported by IPACertTracking
+                flatten = ', '.join("{!s}={!s}".format(key, val)
+                                    for (key, val) in request.items())
+
+                logger.debug(
+                    "Skipping %s since it is handled by IPACertTracking",
+                    flatten
+                )
                 continue
 
             ca_name = certmonger.get_request_value(request_id, 'ca-name')
@@ -566,6 +588,233 @@ class IPACertNSSTrust(IPAPlugin):
                     'verifying trust')
 
 
+@registry
+class IPACertMatchCheck(IPAPlugin):
+    """
+    Ensure certificates match between LDAP and NSS databases
+    """
+
+    requires = ('dirsrv',)
+
+    def get_cert_list_from_db(self, nssdb, nickname):
+        """
+        Retrieve all certificates from an NSS database for nickname.
+        """
+        try:
+            args = ["-L", "-n", nickname, "-a"]
+            result = nssdb.run_certutil(args, capture_output=True)
+            return x509.load_certificate_list(result.raw_output)
+        except ipautil.CalledProcessError:
+            return []
+
+    @duration
+    def check(self):
+        if not self.ca.is_configured():
+            logger.debug("No CA configured, skipping certificate match check")
+            return
+
+        # Ensure /etc/ipa/ca.crt matches the NSS DB CA certificates
+        def match_cacert_and_db(plugin, cacerts, dbpath):
+            db = certs.CertDB(api.env.realm, dbpath)
+            nickname = '%s IPA CA' % api.env.realm
+            try:
+                dbcacerts = self.get_cert_list_from_db(db, nickname)
+            except Exception as e:
+                yield Result(plugin, constants.ERROR,
+                             key=nickname,
+                             error=str(e),
+                             msg='Unable to load CA cert: {error}')
+                return False
+
+            ok = True
+            for cert in dbcacerts:
+                if cert not in cacerts:
+                    ok = False
+                    yield Result(plugin, constants.ERROR,
+                                 key=nickname,
+                                 nickname=nickname,
+                                 serial_number=cert.serial_number,
+                                 dbdir=dbpath,
+                                 certdir=paths.IPA_CA_CRT,
+                                 msg=('CA Certificate nickname {nickname} '
+                                      'with serial number {serial} '
+                                      'is in {dbdir} but is not in'
+                                      '%s' % paths.IPA_CA_CRT))
+            return ok
+
+        try:
+            cacerts = x509.load_certificate_list_from_file(paths.IPA_CA_CRT)
+        except Exception:
+            yield Result(self, constants.ERROR,
+                         key=paths.IPA_CA_CRT.replace(os.path.sep, '_'),
+                         path=paths.IPA_CA_CRT,
+                         msg='Unable to load CA cert file {path}: {error}')
+            return
+
+        # Ensure CA cert entry from LDAP matches /etc/ipa/ca.crt
+        dn = DN('cn=%s IPA CA' % api.env.realm,
+                'cn=certificates,cn=ipa,cn=etc',
+                api.env.basedn)
+        try:
+            entry = self.conn.get_entry(dn)
+        except errors.NotFound:
+            yield Result(self, constants.ERROR,
+                         key=str(dn),
+                         dn=str(dn),
+                         msg='CA Certificate entry \'{dn}\' '
+                             'not found in LDAP')
+            return
+
+        cacerts_ok = True
+        # Are all the certs in LDAP for the IPA CA in /etc/ipa/ca.crt
+        for cert in entry['CACertificate']:
+            if cert not in cacerts:
+                cacerts_ok = False
+                yield Result(self, constants.ERROR,
+                             key=str(dn),
+                             dn=str(dn),
+                             serial_number=cert.serial_number,
+                             msg=('CA Certificate serial number {serial} is '
+                                  'in LDAP \'{dn}\' but is not in '
+                                  '%s' % paths.IPA_CA_CRT))
+
+        # Ensure NSS DBs have matching CA certs for /etc/ipa/ca.crt
+        serverid = realm_to_serverid(api.env.realm)
+        dspath = paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE % serverid
+
+        cacertds_ok = yield from match_cacert_and_db(self, cacerts, dspath)
+        cacertnss_ok = yield from match_cacert_and_db(self, cacerts,
+                                                      paths.IPA_NSSDB_DIR)
+        if cacerts_ok:
+            yield Result(self, constants.SUCCESS,
+                         key=paths.IPA_CA_CRT)
+        if cacertds_ok:
+            yield Result(self, constants.SUCCESS,
+                         key=dspath)
+        if cacertnss_ok:
+            yield Result(self, constants.SUCCESS,
+                         key=paths.IPA_NSSDB_DIR)
+
+
+@registry
+class IPADogtagCertsMatchCheck(IPAPlugin):
+    """
+    Check if dogtag certs present in both NSS DB and LDAP match
+    """
+    requires = ('dirsrv',)
+
+    @duration
+    def check(self):
+        if not self.ca.is_configured():
+            logger.debug('CA is not configured, skipping connectivity check')
+            return
+
+        def match_ldap_nss_cert(plugin, ldap, db, cert_dn, attr, cert_nick):
+            try:
+                entry = ldap.get_entry(cert_dn)
+            except errors.NotFound:
+                yield Result(plugin, constants.ERROR,
+                             key=cert_dn,
+                             msg='%s entry not found in LDAP' % cert_dn)
+                return False
+            try:
+                nsscert = db.get_cert_from_db(cert_nick)
+            except Exception as e:
+                yield Result(plugin, constants.ERROR,
+                             key=cert_nick,
+                             error=str(e),
+                             msg=('Unable to load %s certificate:'
+                                  '{error}' % cert_nick))
+                return False
+            cert_matched = any(cert == nsscert for cert in entry[attr])
+            if not cert_matched:
+                yield Result(plugin, constants.ERROR,
+                             key=cert_nick,
+                             nickname=cert_nick,
+                             dbdir=db.secdir,
+                             msg=('{nickname} certificate in NSS DB {dbdir} '
+                                  'does not match entry in LDAP'))
+                return False
+            return True
+
+        def match_ldap_nss_certs_by_subject(plugin, ldap, db, dn,
+                                            expected_nicks_subjects):
+            entries = ldap.get_entries(dn)
+            all_ok = True
+            for nick, subject in expected_nicks_subjects.items():
+                cert = db.get_cert_from_db(nick)
+                ok = any(
+                    cert in entry["userCertificate"]
+                    and subject == entry["subjectName"][0]
+                    for entry in entries
+                    if "userCertificate" in entry
+                )
+                if not ok:
+                    all_ok = False
+                    yield Result(plugin, constants.ERROR,
+                                 key=nick,
+                                 nickname=nick,
+                                 dbdir=db.secdir,
+                                 msg=('{nickname} certificate in NSS DB '
+                                      '{dbdir} does not match entry in LDAP'))
+            return all_ok
+
+        db = certs.CertDB(api.env.realm, paths.PKI_TOMCAT_ALIAS_DIR)
+        dn = DN('uid=pkidbuser,ou=people,o=ipaca')
+        subsystem_nick = 'subsystemCert cert-pki-ca'
+        subsystem_ok = yield from match_ldap_nss_cert(self, self.conn,
+                                                      db, dn,
+                                                      'userCertificate',
+                                                      subsystem_nick)
+        dn = DN('cn=%s IPA CA' % api.env.realm,
+                'cn=certificates,cn=ipa,cn=etc',
+                api.env.basedn)
+        casigning_nick = 'caSigningCert cert-pki-ca'
+        casigning_ok = yield from match_ldap_nss_cert(self, self.conn,
+                                                      db, dn, 'CACertificate',
+                                                      casigning_nick)
+
+        expected_nicks_subjects = {
+            'ocspSigningCert cert-pki-ca':
+                'CN=OCSP Subsystem,O=%s' % api.env.realm,
+            'subsystemCert cert-pki-ca':
+                'CN=CA Subsystem,O=%s' % api.env.realm,
+            'auditSigningCert cert-pki-ca':
+                'CN=CA Audit,O=%s' % api.env.realm,
+            'Server-Cert cert-pki-ca':
+                'CN=%s,O=%s' % (api.env.host, api.env.realm),
+        }
+
+        kra = krainstance.KRAInstance(api.env.realm)
+        if kra.is_installed():
+            kra_expected_nicks_subjects = {
+                'transportCert cert-pki-kra':
+                    'CN=KRA Transport Certificate,O=%s' % api.env.realm,
+                'storageCert cert-pki-kra':
+                    'CN=KRA Storage Certificate,O=%s' % api.env.realm,
+                'auditSigningCert cert-pki-kra':
+                    'CN=KRA Audit,O=%s' % api.env.realm,
+            }
+            expected_nicks_subjects.update(kra_expected_nicks_subjects)
+
+        ipaca_basedn = DN('ou=certificateRepository,ou=ca,o=ipaca')
+        ipaca_certs_ok = yield from match_ldap_nss_certs_by_subject(
+                                    self, self.conn, db,
+                                    ipaca_basedn,
+                                    expected_nicks_subjects
+                                )
+
+        if subsystem_ok:
+            yield Result(self, constants.SUCCESS,
+                         key=subsystem_nick)
+        if casigning_ok:
+            yield Result(self, constants.SUCCESS,
+                         key=casigning_nick)
+        if ipaca_certs_ok:
+            yield Result(self, constants.SUCCESS,
+                         key=str(ipaca_basedn))
+
+
 @registry
 class IPANSSChainValidation(IPAPlugin):
     """Validate the certificate chain of the certs."""
@@ -593,6 +842,7 @@ class IPANSSChainValidation(IPAPlugin):
             except IOError as e:
                 yield Result(
                     self, constants.ERROR,
+                    key='db_authenticate',
                     error=str(e),
                     msg='Unable to read CA NSSDB token password: {error}')
                 return
@@ -707,6 +957,90 @@ class IPAOpenSSLChainValidation(IPAPlugin):
                         self, constants.SUCCESS, key=cert)
 
 
+def check_agent(plugin, base_dn, agent_type):
+    """Check RA/KRA Agent"""
+
+    try:
+        cert = x509.load_certificate_from_file(paths.RA_AGENT_PEM)
+    except Exception as e:
+        yield Result(plugin, constants.ERROR,
+                     key=paths.RA_AGENT_PEM.replace(os.path.sep, '_'),
+                     error=str(e),
+                     msg='Unable to load RA cert: {error}')
+        return
+    serial_number = cert.serial_number
+    subject = DN(cert.subject)
+    issuer = DN(cert.issuer)
+    description = '2;%d;%s;%s' % (serial_number, issuer, subject)
+    logger.debug('%s agent description should be %s', agent_type, description)
+    db_filter = ldap2.ldap2.combine_filters(
+        [
+            ldap2.ldap2.make_filter({'objectClass': 'inetOrgPerson'}),
+            ldap2.ldap2.make_filter(
+                {'description': ';%s;%s' % (issuer, subject)},
+                exact=False, trailing_wildcard=False),
+        ],
+        ldap2.ldap2.MATCH_ALL)
+    try:
+        entries = plugin.conn.get_entries(base_dn,
+                                          plugin.conn.SCOPE_SUBTREE,
+                                          db_filter)
+    except errors.NotFound:
+        yield Result(plugin, constants.ERROR,
+                     key=agent_type,
+                     description=description,
+                     msg='%s agent not found in LDAP' % agent_type)
+        return
+    except Exception as e:
+        yield Result(plugin, constants.ERROR,
+                     key=agent_type,
+                     error=str(e),
+                     msg='Retrieving %s agent from LDAP failed {error}'
+                         % agent_type)
+        return
+    else:
+        logger.debug('%s agent description is %s', agent_type, description)
+        if len(entries) != 1:
+            yield Result(plugin, constants.ERROR,
+                         key='too_many_agents',
+                         found=len(entries),
+                         msg='Too many %s agent entries found, {found}'
+                             % agent_type)
+            return
+        entry = entries[0]
+        raw_desc = entry.get('description')
+        if raw_desc is None:
+            yield Result(plugin, constants.ERROR,
+                         key='agent_missing_description',
+                         msg='%s agent is missing the description '
+                             'attribute or it is not readable' % agent_type)
+            return
+        ra_desc = raw_desc[0]
+        ra_certs = entry.get('usercertificate')
+        if ra_desc != description:
+            yield Result(plugin, constants.ERROR,
+                         key='description_mismatch',
+                         expected=description,
+                         got=ra_desc,
+                         msg='%s agent description does not match. Found '
+                         '{got} in LDAP and expected {expected}' % agent_type)
+            return
+        found = False
+        for candidate in ra_certs:
+            if candidate == cert:
+                found = True
+                break
+        if not found:
+            yield Result(plugin, constants.ERROR,
+                         key='ldap_mismatch',
+                         certfile=paths.RA_AGENT_PEM,
+                         dn=str(entry.dn),
+                         msg='%s agent certificate in {certfile} not '
+                             'found in LDAP userCertificate attribute '
+                             'for the entry {dn}' % agent_type)
+        yield Result(plugin, constants.SUCCESS)
+
+
 @registry
 class IPARAAgent(IPAPlugin):
     """Validate the RA Agent used to talk to the CA
@@ -722,82 +1056,32 @@ class IPARAAgent(IPAPlugin):
             logger.debug('CA is not configured, skipping RA Agent check')
             return
 
-        try:
-            cert = x509.load_certificate_from_file(paths.RA_AGENT_PEM)
-        except Exception as e:
-            yield Result(self, constants.ERROR,
-                         error=str(e),
-                         msg='Unable to load RA cert: {error}')
+        base_dn = DN('uid=ipara,ou=people,o=ipaca')
+        yield from check_agent(self, base_dn, 'RA')
+
+
+@registry
+class IPAKRAAgent(IPAPlugin):
+    """Validate the KRA Agent
+
+       Compare the description and usercertificate values.
+    """
+
+    requires = ('dirsrv',)
+
+    @duration
+    def check(self):
+        if not self.ca.is_configured():
+            logger.debug('CA is not configured, skipping KRA Agent check')
             return
 
-        serial_number = cert.serial_number
-        subject = DN(cert.subject)
-        issuer = DN(cert.issuer)
-        description = '2;%d;%s;%s' % (serial_number, issuer, subject)
-
-        logger.debug('RA agent description should be %s', description)
-
-        db_filter = ldap2.ldap2.combine_filters(
-            [
-                ldap2.ldap2.make_filter({'objectClass': 'inetOrgPerson'}),
-                ldap2.ldap2.make_filter({'sn': 'ipara'}),
-                ldap2.ldap2.make_filter(
-                    {'description': ';%s;%s' % (issuer, subject)},
-                    exact=False, trailing_wildcard=False),
-            ],
-            ldap2.ldap2.MATCH_ALL)
-
-        base_dn = DN(('o', 'ipaca'))
-        try:
-            entries = self.conn.get_entries(base_dn,
-                                            self.conn.SCOPE_SUBTREE,
-                                            db_filter)
-        except errors.NotFound:
-            yield Result(self, constants.ERROR,
-                         description=description,
-                         msg='RA agent not found in LDAP')
+        kra = krainstance.KRAInstance(api.env.realm)
+        if not kra.is_installed():
+            logger.debug('KRA is not installed, skipping KRA Agent check')
             return
-        except Exception as e:
-            yield Result(self, constants.ERROR,
-                         error=str(e),
-                         msg='Retrieving RA agent from LDAP failed {error}')
-            return
-        else:
-            logger.debug('RA agent description is %s', description)
-            if len(entries) != 1:
-                yield Result(self, constants.ERROR,
-                             found=len(entries),
-                             msg='Too many RA agent entries found, {found}')
-                return
-            entry = entries[0]
-            raw_desc = entry.get('description')
-            if raw_desc is None:
-                yield Result(self, constants.ERROR,
-                             msg='RA agent is missing the description '
-                                 'attribute or it is not readable')
-                return
-            ra_desc = raw_desc[0]
-            ra_certs = entry.get('usercertificate')
-            if ra_desc != description:
-                yield Result(self, constants.ERROR,
-                             expected=description,
-                             got=ra_desc,
-                             msg='RA agent description does not match. Found '
-                             '{got} in LDAP and expected {expected}')
-                return
-            found = False
-            for candidate in ra_certs:
-                if candidate == cert:
-                    found = True
-                    break
-            if not found:
-                yield Result(self, constants.ERROR,
-                             certfile=paths.RA_AGENT_PEM,
-                             dn=str(entry.dn),
-                             msg='RA agent certificate in {certfile} not '
-                                 'found in LDAP userCertificate attribute '
-                                 'for the entry {dn}')
-            yield Result(self, constants.SUCCESS)
+
+        base_dn = DN('uid=ipakra,ou=people,o=kra,o=ipaca')
+        yield from check_agent(self, base_dn, 'KRA')
 
 
 @registry
